@@ -2,8 +2,8 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -11,20 +11,20 @@
 #include "header.h"
 
 
-struct window *cwwindow;
+//static struct window *cwwindow;
 
-void draw_cwwindow(struct cwwin_data *cwwdata){
+static void draw_cwwindow(struct cwwin_data *cwwdata){
     fill_area(cwwdata->x-2, cwwdata->y-1, cwwdata->w+4, cwwdata->h+2, COL_INV); 
     draw_frame(cwwdata->x, cwwdata->y, cwwdata->w, cwwdata->h, COL_INV, 1); 
     print_text(cwwdata->x + 5, cwwdata->y, -1, VTEXT(T__CW_), COL_NORM);
     print_text(cwwdata->x + 5, cwwdata->y + cwwdata->h -1 , -1, VTEXT(T_CWW_KEYS), COL_NORM);
 
-    print_text(cwwdata->x + 3, cwwdata->y +2 , CWW_MAXSTR>cwwdata->w-4? cwwdata->w-4:CWW_MAXSTR, cwwdata->text , COL_INV);
+    print_text(cwwdata->x + 3, cwwdata->y +2 , cwwdata->w-4, cwwdata->text->str , COL_INV);
         
     /*dbg("draw_cwwindow: set_cursor %d %d \n", cwwdata->x +
 	 * strlen(cwwdata->text), cwwdata->y+2, 0, 0);*/
-    set_cursor(cwwdata->x + strlen(cwwdata->text) + 3, cwwdata->y+2, 
-                     cwwdata->x + strlen(cwwdata->text) + 3 , cwwdata->y+2);
+    set_cursor(cwwdata->x + cwwdata->text->len + 3, cwwdata->y+2, 
+                     cwwdata->x + cwwdata->text->len + 3 , cwwdata->y+2);
     
 }
 
@@ -35,12 +35,13 @@ void cwwindow_func(struct window *win, struct event *ev, int fwd){
     term = win->term;
     cwwdata = (struct cwwin_data *)win->data;
     
-    dbg("cwwindow_func [%d,%d,%d,%d]\n",ev->ev,ev->x,ev->y,ev->b);
+//    dbg("cwwindow_func [%d,%d,%d,%d]\n",ev->ev,ev->x,ev->y,ev->b);
 
     switch (ev->ev) {
         case EV_INIT:
             win->data = cwwdata = mem_alloc(sizeof(struct cwwin_data));
             memset(win->data, 0, sizeof(struct cwwin_data));
+			cwwdata->text = g_string_sized_new(100);
 
         case EV_RESIZE:
             cwwdata->x = 10;
@@ -74,20 +75,34 @@ void cwwindow_func(struct window *win, struct event *ev, int fwd){
 					redraw_later();
                     break;  
                 default:    
-                    if (ev->x >= ' ' && ev->x < 256) {
-						char s[2];
-                        dbg("key=%c\n", ev->x);
-                        if (strlen(cwwdata->text)<CWW_MAXSTR-1)
-                            cwwdata->text[strlen(cwwdata->text)]=ev->x;
-						s[0]=ev->x;
-						s[1]='\0';
-                        cwdaemon_text(cwda, s);
-                        draw_cwwindow(cwwdata);
+                    if (ev->x == KBD_BS && ev->y == 0){
+                        if (cwda && cwda->back) cwda->back(cwda);
+                        if (cwwdata->text->len > 0)
+                            g_string_truncate(cwwdata->text, cwwdata->text->len - 1);
+                        redraw_later();
+                        break;
                     }
+                    if (ev->x >= ' ' && ev->x < 256 && ev->y == 0) {
+						char s[2];
+						int n;
+                        dbg("key=%c\n", ev->x);
+						
+						g_string_append_c(cwwdata->text, (char)ev->x);
+						n = cwwdata->text->len - (cwwdata->w - 7);
+						if (n>0) g_string_erase(cwwdata->text, 0, n);
+						s[0]=(char)ev->x;
+						s[1]='\0';
+                        //cwdaemon_text(cwda, s);
+                        if (cwda && cwda->text) cwda->text(cwda, s);
+                        draw_cwwindow(cwwdata);
+                        break;
+                    }
+                    break;
             }
             break;
         case EV_ABORT:
             /* mem_free(cwwdata);  already in terminal.c:196 (delete_window) */ 
+			g_string_free(cwwdata->text, 1);
             break;
     }
 }
