@@ -2,16 +2,17 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
 
-#include <header.h>
-void dxc_dump(struct spotdb *spotdb, int bi);
+#include "header.h"
 
-#define MIN_SPOTLEN 74
+
+//#define MIN_SPOTLEN 74
+#define MIN_SPOTLEN 29
 #define STR_DX_DE "DX de"
 #define FROM_OFS 6
 #define CALL_OFS 26
@@ -24,12 +25,12 @@ void dxc_dump(struct spotdb *spotdb, int bi);
 struct spotdb *spotdb=NULL;
 
 
-struct spotdb *init_spotdb(){
+struct spotdb *init_spotdb(void){
     struct spotdb *spotdb;
 
     spotdb=g_new0(struct spotdb, 1);
-    spotdb->first=g_new0(struct spot *, cfg->bands->len);
-    spotdb->cur  =g_new0(struct spot *, cfg->bands->len);
+    spotdb->first=g_new0(struct spot *, 'Z'-'A'+1);
+    spotdb->cur  =g_new0(struct spot *, 'Z'-'A'+1);
     spotdb->timer_id=install_timer(DXC_EXPIRE_TIMER, dxc_timer, CBA0);
     return spotdb;
 }
@@ -67,7 +68,7 @@ void free_spotdb(struct spotdb *spotdb){
     g_free(spotdb);
 }
 
-void dxc_timer(union cba_t cba){
+void dxc_timer(cba_t cba){
     int n;
     n=dxc_remove_expired();
     /*dbg("dxc_timer expired=%d\n",n);*/
@@ -86,6 +87,15 @@ struct spot *dxc_parse_spot(gchar *str){
     struct spot *spot;
     gchar *c,*d;
     time_t now;
+	char *c1=NULL, *c2=NULL;
+
+
+   // dbg("dxc_parse_spot('%s')]\n", str);
+	if (regmatch(str, "SFI=([0-9]{1,3})", &c1, &c2, NULL)==0){
+		spotdb->sfi = atoi(c2);
+	}
+	if (c1) mem_free(c1);
+	if (c2) mem_free(c2);
 
     
     if (strlen(str)<MIN_SPOTLEN) return NULL;
@@ -94,7 +104,7 @@ struct spot *dxc_parse_spot(gchar *str){
     callsign=from=text=NULL;
     
     d=str+FROM_OFS;
-    c=index(d,FROM_SEP);
+    c=strchr(d,FROM_SEP);
     
     if (!c) goto x;
     from=g_strndup(d, c-d);
@@ -104,21 +114,35 @@ struct spot *dxc_parse_spot(gchar *str){
     qrg=strtod(c, (char **)NULL);
 
     d=str+CALL_OFS;
-    c=index(d,' ');
+    c=strchr(d,' ');
     if (!c) goto x;
     callsign=g_strndup(d, c-d);
     if (strlen(callsign)<3) goto x;
 
-    d=str+ZULU_OFS;
-    d[4]='\0';
-    zulu=atoi(d);
+    if (strlen(str)>ZULU_OFS){
+        d=str+ZULU_OFS;
+        d[4]='\0';
+        zulu=atoi(d);
+    }else{
+        time_t now;
+        struct tm tm;
+        now = time(NULL);
+        gmtime_r(&now, &tm);
+        zulu = tm.tm_hour * 100 + tm.tm_min;
+    }
 
-    d=str+TEXT_OFS;
-    str[ZULU_OFS]='\0';
-    c=rindex(d, ' ');
-    g_strstrip(d);
-    if (!c) c=d+ZULU_OFS-1;
-    text=g_strndup(d, c-d);
+    if (strlen(str)>TEXT_OFS){
+        d=str+TEXT_OFS;
+        if (strlen(str)>ZULU_OFS){
+            str[ZULU_OFS]='\0';
+        }
+        c=strrchr(d, ' ');
+        g_strstrip(d);
+        if (!c) c=d+ZULU_OFS-1;
+        text=g_strndup(d, c-d);
+    }else{
+        text=g_strdup("");
+    }
 
     now=time(NULL);
     
@@ -136,8 +160,7 @@ struct spot *dxc_parse_spot(gchar *str){
     spot->expire  =now + 2 * 60;
 #endif    
 
-    /*dbg("call='%s' qrg='%3.1f' from='%s' text='%s' zulu='%d'\n",
-            callsign, qrg, from, text, zulu);*/
+//    dbg("call='%s' qrg='%3.1f' from='%s' text='%s' zulu='%d'\n", callsign, qrg, from, text, zulu);
     return spot;    
 x:;
     CONDGFREE(callsign);
@@ -189,24 +212,24 @@ int dxc_read_spot(gchar *str){
     if (!newspot) return -3;
     
     /*dbg(" spot accepted %p\n", str, newspot);*/
-    confb=get_config_band_by_qrg(newspot->qrg);
+    confb=(struct config_band*)get_config_band_by_qrg((int)newspot->qrg);
     if (!confb){
         free_spot(newspot);
-        /*dbg(" bad qrg\n");*/
+        dbg(" bad qrg\n");
         return -1;
     }
     
-    bi=lowcase(confb->bandchar)-'a';
-    if (bi<0 || bi>=cfg->bands->len){
+    if (confb->bandchar<'a' || confb->bandchar>'z'){
         free_spot(newspot);
-/*        dbg(" bad band\n");   */
-        return -2;
+        dbg(" bad band\n");   
+        return 0;
     }
+    bi=lowcase(confb->bandchar)-'a';
     
     now=time(NULL);
     newpos=NULL;
     for (spot=spotdb->first[bi]; spot; ){
-        /*dbg("spot=%s qrg=%3.1f\n", spot->callsign, spot->qrg);*/
+        dbg("spot=%s qrg=%3.1f\n", spot->callsign, spot->qrg);
         if (strcasecmp(spot->callsign, newspot->callsign)==0 ||
             spot->expire < now) {
             struct spot *tofree;
@@ -261,7 +284,7 @@ int dxc_read_spot(gchar *str){
     if (bi==8){dbg("b: %p\n", spotdb->cur[bi]); }
     dxc_assert(spotdb, bi);
 #endif    
-    return spot!=NULL;
+    return 0;
 }
 
 int dxc_remove_expired(void){
