@@ -2,8 +2,8 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -13,7 +13,7 @@
 
 struct fifo *glog = NULL;
 struct fifo *gtalk = NULL;
-struct fifo *gsked = NULL;
+//static struct fifo *gsked = NULL;
 
 struct fifo *init_fifo(int maxlen){
     struct fifo *fifo;
@@ -58,7 +58,7 @@ gchar *fifo_index(struct fifo *fifo, int index){
 
 
 
-void fifo_check_len(struct fifo *fifo){
+static void fifo_check_len(struct fifo *fifo){
     gpointer p;
     
     if (fifo->items->len == fifo->maxlen){
@@ -69,7 +69,7 @@ void fifo_check_len(struct fifo *fifo){
 }
 
 
-gchar *hour_min(void){
+static gchar *hour_min(void){
     time_t now;
     struct tm utc;
     gchar *res;
@@ -77,7 +77,7 @@ gchar *hour_min(void){
     time(&now);
     gmtime_r(&now, &utc);
 
-    res=g_strdup_printf("%2d.%02d",utc.tm_hour, utc.tm_min);
+    res=g_strdup_printf("%2d:%02d",utc.tm_hour, utc.tm_min);
     return res;
 }
 
