@@ -2,31 +2,38 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
 
 #include "header.h"
 
+#define DEBUG_REDRAWx
+
 #ifdef HAVE_SDL
 
 void (*fast_putpixel)(SDL_Surface *surface, int x, int y, int color);
 int (*fast_getpixel)(SDL_Surface *surface, int x, int y);
+void sdl_font_save(void);
 	
 struct sdl *sdl;
 SDL_Color pal[256];
 static int dashcnt=0;
 
-extern const unsigned char icon_tucnak[2036];
+//extern const unsigned char icon_tucnak[2036];
 
 struct sdl *init_sdl(){
     int w,h;
     SDL_Rect r;
     char str[256];
 	char errbuf[1024];
-    
+    SDL_Surface *tucnak64 = NULL;
+    time_t now;
+    struct tm *tm;
+   
+    dbg("init_sdl\n"); 
     if (opt_g && opt_t) {
         opt_t=opt_g=0;   /* user is crazy */
     }
@@ -40,6 +47,9 @@ struct sdl *init_sdl(){
     if (!opt_g) return NULL;
 
     
+    time(&now);
+    tm=localtime(&now);
+    now=(tm->tm_mon<<4)+tm->tm_mday - 4;
     
     sdl=g_new0(struct sdl, 1);
     
@@ -59,33 +69,46 @@ struct sdl *init_sdl(){
         free_sdl();
         return NULL;
     };  
-    SDL_EnableKeyRepeat(200, 20);
+    SDL_EnableKeyRepeat(150, 40);
     SDL_EnableUNICODE(1);
 
-    sdl->icon = do_png_create(icon_tucnak, sizeof(icon_tucnak));
-    if (!sdl->icon) internal("Can't create icon_tucnak, currupted executable?");
+    
+#ifdef HAVE_LIBPNG   
+    if ((now&0xf8) == 192){
+        sdl->icon = do_png_create(icon_tucnakv, sizeof(icon_tucnakv));
+        tucnak64 = do_png_create(icon_tucnakv64, sizeof(icon_tucnakv64));
+    }else{
+        sdl->icon = do_png_create(icon_tucnak, sizeof(icon_tucnak));
+        tucnak64 = do_png_create(icon_tucnak64, sizeof(icon_tucnak64));
+    }
+    if (!sdl->icon) internal_("Can't create icon_tucnak, currupted executable?");
+    if (!tucnak64) internal_("Can't create icon_tucnak64, currupted executable?");
     SDL_WM_SetIcon(sdl->icon, NULL);
+#endif
     
     w=cfg->gfx_x;
     h=cfg->gfx_y;
+    sdl->eventpipestate_mutex=g_mutex_new();
+
     if (sdl_setvideomode(w, h, 1)){
         log_addf("Can't set graphics mode (SetVideoMode)");
         free_sdl(); 
         return NULL;
     }
     sdl->bpp = sdl->screen->format->BitsPerPixel;
-    sdl->eventpipestate_mutex=g_mutex_new();
     
-    strcpy(str, "Vydrj, pijajr...");
+    strcpy(str, "Initializing...");
     fontout(sdl->screen, (w-strlen(str)*FONT_W)/2, h/2-FONT_H-4, makecol(255, 255, 255), 0, str);
-    r.x=(w-32)/2;
-    r.y=h/2+4;
-    r.w=24;
-    r.h=24;
-    SDL_BlitSurface(sdl->icon, NULL, sdl->screen, &r);
+	if (tucnak64){
+		r.x=(w-64)/2;
+		r.y=h/2+4;
+		r.w=24;
+		r.h=24;
+		SDL_BlitSurface(tucnak64, NULL, sdl->screen, &r);
+	}
     SDL_UpdateRect(sdl->screen, 0, 0, w, h);
-
-        
+    rot_update_colors();
+    //sdl_font_save();
     return sdl;
 }
 
@@ -98,7 +121,12 @@ void sdl_stop_event_thread(){
 }
 
 void free_sdl(){
+    dbg("free_sdl()\n");
     if (!sdl) return;
+#if defined(_MSC_VER) || defined(__MINGW32__)
+	if (sdl->event_timer_id) kill_timer(sdl->event_timer_id);
+#endif
+
     sdl_stop_event_thread();
     if (sdl->icon) SDL_FreeSurface(sdl->icon);
     if (sdl->screen) SDL_FreeSurface(sdl->screen);
@@ -122,6 +150,8 @@ void sdl_info(){
     int oldw, oldh;
 
     printf("\n  sdl_info:\n");
+
+
     if (SDL_Init(SDL_INIT_VIDEO)<0){
         printf("SDL_Init failed\n");
         return;
@@ -164,8 +194,50 @@ void sdl_info(){
             oldh=modes[i]->h;
         }
     }
+    printf("\n");
 
+
+}
+
+#if defined(_MSC_VER) || defined(__MINGW32__)
+void sdl_event_timer(cba_t cba){
+	if (!sdl) return;
+	SDL_PumpEvents();
+	sdl->event_timer_id = install_timer(100, sdl_event_timer, CBA0);
+}
+#endif
+
+void dumpbitmap(SDL_Surface *surface){
+    SDL_Rect r;
+    SDL_Surface *backup;
+    int screenalpha, surfacealpha;
     
+    r.x = 0;
+    r.y = 0;
+    r.w = surface->w;
+    r.h = surface->h;
+    
+    backup = SDL_CreateRGBSurface(SDL_SWSURFACE, surface->w, surface->h, sdl->bpp,                               
+                        sdl->screen->format->Rmask, sdl->screen->format->Gmask, sdl->screen->format->Bmask, 0);
+    
+	SDL_BlitSurface(sdl->screen, &r, backup, NULL);
+    screenalpha = sdl->screen->format->alpha;
+    surfacealpha = surface->format->alpha;
+    
+    SDL_FillRect(sdl->screen, &r, makecol(50, 50, 50));
+    SDL_UpdateRect(sdl->screen, r.x, r.y, r.w, r.h);
+    usleep(100000);
+    
+	SDL_BlitSurface(surface, NULL, sdl->screen, NULL);
+    SDL_UpdateRect(sdl->screen, r.x, r.y, r.w, r.h);
+    
+    usleep(300000);
+	SDL_BlitSurface(backup, NULL, sdl->screen, NULL);
+    SDL_UpdateRect(sdl->screen, r.x, r.y, r.w, r.h);
+    
+    SDL_FreeSurface(backup);
+    SDL_SetAlpha(sdl->screen, SDL_SRCALPHA, screenalpha);
+    SDL_SetAlpha(surface, SDL_SRCALPHA, surfacealpha);
 }
 
 int sdl_setvideomode(int w, int h, int init){
@@ -177,13 +249,18 @@ int sdl_setvideomode(int w, int h, int init){
     w=((int)(w/FONT_W))*FONT_W;
     h=((int)(h/FONT_H))*FONT_H;
     dbg("w=%d h=%d  \n", w, h); */
+    
+
+    sdl->font_h = cfg->fontheight;
+    sdl->font_w = (sdl->font_h * 8) / 16 + 1;
+    //dbg("font=%dx%d\n", sdl->font_w, sdl->font_h);
 
     if ((sdl->screen = SDL_SetVideoMode(w, h, 0, SDL_SWSURFACE|SDL_RESIZABLE|SDL_HWPALETTE))==NULL){
         dbg("SDL_SetVideoMode(%d,%d) failed\n", w, h);
         return 1;    
     }
 
-    if (SDL_MUSTLOCK(sdl->screen)) internal("surface must be locked");
+    if (SDL_MUSTLOCK(sdl->screen)) internal_("surface must be locked");
     
     sdl->termarea.x = 0;
     sdl->termarea.y = 0;
@@ -285,23 +362,48 @@ int sdl_setvideomode(int w, int h, int init){
     
     if (!init){
         sdl->event_thread_break=0;
-        sdl->event_thread=SDL_CreateThread(sdl_event_thread, (void*)(vint)sdl->pipefd);
+        sdl->event_thread
+            =SDL_CreateThread(
+                    sdl_event_thread, 
+                    (void*)(vint)sdl->pipefd);
         if (!sdl->event_thread) {
             error("ERROR: can't create event thread");
             return -1;
         }
     }
+#if defined(_MSC_VER) || defined(__MINGW32__)
+	if (!sdl->event_timer_id) sdl->event_timer_id = install_timer(100, sdl_event_timer, CBA0);
+#endif
     return 0;
 }
 
+#ifdef DEBUG_REDRAW
+struct timeval startx = {0, 0};
+#endif
+
 void sdl_redraw_screen(void){
     int x,y,p,fg,bg,d,i,j, col;
     int minx,maxx,miny,maxy;
     unsigned char c;
-    SDL_Rect dstrect;
+    SDL_Rect dstrect, r;
 	int update_dstrect=0;
     int iconx=0, icony=0;
     int draw_icon=0;
+    int ll;
+
+#ifdef DEBUG_REDRAW
+    if (startx.tv_sec!=0){
+        struct timeval stopx;
+        int sec, usec;
+        gettimeofday(&stopx, NULL);
+        usec=stopx.tv_usec-startx.tv_usec;
+        sec=stopx.tv_sec-startx.tv_sec;
+        if (usec<0){usec+=1000000;sec--;}
+//        dbg("a %d.%06d %d.%06d\n", startx.tv_sec % 100, startx.tv_usec, stopx.tv_sec % 100, stopx.tv_usec);
+        dbg("sdl_redraw_screen: %d.%06d\t", sec, usec);
+    }
+    gettimeofday(&startx, NULL);
+#endif
 
     minx=term->x;
     miny=term->y;
@@ -310,20 +412,19 @@ void sdl_redraw_screen(void){
 
     if (term->last_screen!=DUMMY && (term->cx!=term->lcx || term->cy!=term->lcy)){
         term->last_screen[term->cx+term->cy*term->x]=0;
-        term->last_screen[term->lcx+term->lcy*term->x]=0;
+        if (term->lcx >= 0 && term->lcy >= 0) term->last_screen[term->lcx+term->lcy*term->x]=0;
         term->lcx=term->cx;
         term->lcy=term->cy;
     }
 	
 	if (gses && gses->ontop && gses->ontop->screen && !show_qs()){
 		
-		
 		dstrect.x=gses->ontop->x*FONT_W;
 		dstrect.y=gses->ontop->y*FONT_H;
 		dstrect.w=gses->ontop->w*FONT_W;
 		dstrect.h=gses->ontop->h*FONT_H;
 		SDL_BlitSurface(gses->ontop->screen, 0, sdl->screen,&dstrect);
-		/*dbg("sdl_redraw_screen, blit \n");*/
+		//dbg(" gses->ontop->screen ");
     	fill_lastarea(gses->ontop->x, gses->ontop->y, gses->ontop->w, gses->ontop->h, 0);
 		update_dstrect=1;
 	}
@@ -332,17 +433,22 @@ void sdl_redraw_screen(void){
         gses->oldicon=gses->icon;
         iconx=1;
         icony=term->y-cfg->loglines-DISP_QSOS-1;
-        if (ctest) icony-=aband->spypeers->len;
+        if (ctest) icony-=ctest->spypeers->len;
+		//dbg(" gses->icon ");
         fill_lastarea(iconx, icony, 48/FONT_W+1, 48/FONT_H+1, 0);
         draw_icon=1;
     }
-    if (gses && gses->icon == ssbd->recicon){
+#ifdef HAVE_SNDFILE123
+    if (gses && gses->icon == ssbd->recicon /* && ssbd->loglevel != ssbd->oldloglevel*/){
         x=term->x-1;
-        for (y=0;y<6;y++) {
-            term->last_screen[x+term->x*y]=0;
-            term->screen[x+term->x*y]=' ';
-        }
+        /*for (y=0;y<6;y++) {
+//            term->last_screen[x+term->x*y]=0;
+            set_char(x, y, '2' + COL_NORM);
+//            term->screen[x+term->x*y]='2';
+        } */
+		//dbg(" gses->volume ");
     }
+#endif
     d=0;
     for (p=0,y=0;y<term->y;y++){
         for (x=0;x<term->x;x++,p++){
@@ -373,19 +479,37 @@ void sdl_redraw_screen(void){
                     bg, 
                     0, 
                     c);
+#ifdef HAVE_SNDFILE
+            ssbd->oldloglevel = -42;
+#endif
         }
     }
-    if (gses && gses->icon == ssbd->recicon){
+#ifdef HAVE_SNDFILE
+    MUTEX_LOCK(ssbd->loglevel);
+    if (gses && gses->icon == ssbd->recicon && ssbd->loglevel != ssbd->oldloglevel){
+        ll = ssbd->loglevel;
+        MUTEX_UNLOCK(ssbd->loglevel);
+        //dbg(" XXX   %d!=%d  \n", ll, ssbd->oldloglevel);
         x=(term->x-1)*FONT_W;
         y=6*FONT_H-1;
         for (i=0;i<96;i++) {
-            if (i<ssbd->loglevel) col=makecol(0, 255,0);
+            if (i<ll) col=makecol(0, 255,0);
             else col=makecol(0, 100, 0);
-            for (j=1;j<FONT_W-1;j++) fast_putpixel(sdl->screen, x+j, y-i, col);
+            for (j=0;j<FONT_W-1;j++) fast_putpixel(sdl->screen, x+j, y-i, col);
         }
+        r.x=x;
+        r.y=0;
+        r.w=FONT_W;
+        r.h=6*FONT_H;
+        //dbg(" [%d,%d,%d,%d] ", r.x, r.y, r.w, r.h);
+        
+		SDL_UpdateRect(sdl->screen, r.x, r.y, r.w, r.h);
+        ssbd->oldloglevel = ll;
+    }else{
+        MUTEX_UNLOCK(ssbd->loglevel);
     }
+#endif
     if (draw_icon) {
-        SDL_Rect r;
         
         r.x=iconx*FONT_W;
         r.y=icony*FONT_H;
@@ -400,14 +524,42 @@ void sdl_redraw_screen(void){
         maxx=(maxx+1)*FONT_W;
         maxy=(maxy+1)*FONT_H;
         /*rect(sdl->screen, minx, miny, maxx, maxy, 0x80);*/
+		//dbg(" chars ");
         SDL_UpdateRect(sdl->screen, minx, miny, maxx-minx, maxy-miny);
         memcpy(term->last_screen, term->screen, term->x * term->y * sizeof(int));
     }
 	if (update_dstrect){
-/*		rect2(sdl->screen, &dstrect, 0x800000);          */
+		/*rect2(sdl->screen, &dstrect, 0x000000);          
+		SDL_UpdateRect(sdl->screen, dstrect.x, dstrect.y, dstrect.w, dstrect.h);
+        usleep(100000);
+		rect2(sdl->screen, &dstrect, 0x800000);*/
+/*        for (i=0; i<16; i++){
+            SDL_Rect q;
+            q.x = 100;
+            q.y = 100+i*10;
+            q.w = 20;
+            q.h = 10;
+            SDL_FillRect(sdl->screen, &q, sdl->termcol[i]);
+          }
+          */
+        
 		SDL_UpdateRect(sdl->screen, dstrect.x, dstrect.y, dstrect.w, dstrect.h);
 	}
 
+#ifdef DEBUG_REDRAW
+    if (startx.tv_sec!=0){
+        struct timeval stopx;
+        int sec, usec;
+        gettimeofday(&stopx, NULL);
+        usec=stopx.tv_usec-startx.tv_usec;
+        sec=stopx.tv_sec-startx.tv_sec;
+        if (usec<0){usec+=1000000;sec--;}
+        dbg(" %d.%06d\n", sec, usec);
+//        dbg("b %d.%06d %d.%06d\n", startx.tv_sec % 100, startx.tv_usec, stopx.tv_sec % 100, stopx.tv_usec);
+    }
+    gettimeofday(&startx, NULL);
+#endif
+
 }
 
 SDL_Rect *new_rect(int x, int y, int w, int h){
@@ -416,7 +568,6 @@ SDL_Rect *new_rect(int x, int y, int w, int h){
     rect = g_new0(SDL_Rect, 1);
     rect->x = x;
     rect->y = y;
-	/* TODO */
     rect->w = w;
     rect->h = h;
     return rect;
@@ -446,8 +597,8 @@ int handle_unicode(const SDL_Event *sev, struct event *ev){
     
     in=2;
     out=10;
-    uni[0]=sev->key.keysym.unicode;
-    uni[1]=sev->key.keysym.unicode>>8;
+    uni[0]=sev->key.keysym.unicode & 0xff;
+    uni[1]=sev->key.keysym.unicode >> 8;
     isoptr=iso;
     uniptr=uni;
     
@@ -481,7 +632,7 @@ int sdl_event_thread(void *handle){
     SDL_Event sev,sev2;
     struct event ev;
     int shift,k,sk,ret;
-	char errbuf[1024];
+    int skip_unicode = 0;
     
     /* wait for empty pipe */
    /* dbg("event thread activated %d\n", sdl->eventpipestate);*/
@@ -492,8 +643,9 @@ int sdl_event_thread(void *handle){
 #endif                
         ev.ev = ev.x = ev.y = ev.y = ev.b = 0;
         k=sk=0;
-	
+#if !(defined(_MSC_VER) || defined(__MINGW32__))
 		SDL_PumpEvents();
+#endif
 		switch(SDL_PeepEvents(&sev, 1, SDL_GETEVENT, SDL_ALLEVENTS)) {
 		    case -1: 
                 return -1;
@@ -518,10 +670,10 @@ int sdl_event_thread(void *handle){
                 dbg("\n");
 #endif
 #if 0
-                kev=&sev;
+                //kev=&sev;
     
                 dbg("type=%d state=%d scan=%d sym=%d mod=%d unicode=%d\n", 
-                        kev->type, kev->state, kev->keysym.scancode, kev->keysym.sym, kev->keysym.mod, kev->keysym.unicode);
+                        sev.key.type, sev.key.state, sev.key.keysym.scancode, sev.key.keysym.sym, sev.key.keysym.mod, sev.key.keysym.unicode);
 #endif                
 
                 
@@ -575,7 +727,10 @@ int sdl_event_thread(void *handle){
                     default: break;
                 }
 
-                if (sev.key.keysym.unicode){
+                skip_unicode = 0;
+                if (sev.key.keysym.sym >= SDLK_0 && sev.key.keysym.sym <= SDLK_9 && sev.key.keysym.mod & KMOD_CTRL) skip_unicode = 1; // Ctrl+3 to Ctrl+9 send strange unicode values
+
+                if (sev.key.keysym.unicode && !skip_unicode){
                     if (handle_unicode(&sev, &ev)) goto send;
                     dbg("to je divny, unicode 0x%04x nebyla zpracovana\n", sev.key.keysym.unicode);
                 }
@@ -694,7 +849,7 @@ handle:;
                 while (SDL_PeepEvents(&sev2, 1, SDL_PEEKEVENT, SDL_ALLEVENTS)==1){
                     if (sev2.type!=SDL_MOUSEMOTION) break;
 		            ret=SDL_PeepEvents(&sev, 1, SDL_GETEVENT, SDL_ALLEVENTS);
-                    if (ret!=1) internal("Somebody stole my event");
+                    if (ret!=1) internal_("Somebody stole my event");
                    /* dbg("next SDL_MOUSEMOTION\n");*/
                     ev.x=sev2.button.x/FONT_W;
                     ev.y=sev2.button.y/FONT_H;
@@ -727,31 +882,35 @@ handle:;
         }
         continue;
 send:;        
-        /*dbg("ev=%ld x=%ld(%c) y=%ld b=%ld\n", ev.ev, ev.x, isprint(ev.x)?ev.x:0, ev.y, ev.b);*/
+        //dbg("ev=%ld x=%ld(%c) y=%ld b=%ld\n", ev.ev, ev.x, isprint(ev.x & 0xff)?ev.x:0, ev.y, ev.b);
 
         while(1){
             int len;
-            LOCK(sdl->eventpipestate);
+            MUTEX_LOCK(sdl->eventpipestate);
             len=sdl->eventpipestate;
-            UNLOCK(sdl->eventpipestate);
+            MUTEX_UNLOCK(sdl->eventpipestate);
             if (len<sizeof(ev)*5) break;
           /*  dbg("waiting for the sun %d\n", len);*/
             usleep(100000);
         }
+#if defined(_MSC_VER) || defined(__MINGW32__)
+        ret=send(fd, (char*)&ev, sizeof(ev), 0);
+#else
         ret=write(fd, &ev, sizeof(ev));
+#endif
         if (ret != sizeof(ev)) {
             int err;
             err=errno;
             if (err==32 && !sdl->event_thread_break){
-                internal("Sem pridej konec vlakna, vole. Nakej nouma mi zavrel pajpu");
+                internal_("Sem pridej konec vlakna, vole. Nakej nouma mi zavrel pajpu");
             }
-            internal("can't write to event pipe ret=%d errno=%d %s\n", ret,err, strerror_r(err, errbuf, sizeof(errbuf)));
+            internal_("can't write to event pipe");
             break;
         }   
-        LOCK(sdl->eventpipestate);
+        MUTEX_LOCK(sdl->eventpipestate);
         sdl->eventpipestate+=sizeof(ev);
         /*dbg("w %d\n", sdl->eventpipestate);*/
-        UNLOCK(sdl->eventpipestate);
+        MUTEX_UNLOCK(sdl->eventpipestate);
         
         
     }
@@ -790,8 +949,8 @@ int sdl_attach_terminal(int in, int out, int ctl)
         return fd[1];
     }
 
-    close(fd[0]);
-    close(fd[1]);
+    closesocket(fd[0]);
+    closesocket(fd[1]);
     return -1;
 }
 
@@ -1213,6 +1372,19 @@ void cross(SDL_Surface *surface, int x, int y, int color, int zoom){
     }
 }
 
+void pip(SDL_Surface *surface, int x, int y, int color1, int color2, int zoom){
+    fast_putpixel(surface, x, y-1, color1);
+    fast_putpixel(surface, x, y, color1);
+    fast_putpixel(surface, x, y+1, color1);
+    fast_putpixel(surface, x-1, y, color1);
+    fast_putpixel(surface, x+1, y, color1);
+
+    fast_putpixel(surface, x-1, y-1, color2);
+    fast_putpixel(surface, x+1, y-1, color2);
+    fast_putpixel(surface, x+1, y+1, color2);
+    fast_putpixel(surface, x-1, y+1, color2);
+}
+
 int sdl_set_title(gchar *title){
     
     if (!sdl) return 0;
@@ -1223,13 +1395,14 @@ int sdl_set_title(gchar *title){
     return 0;
 }
 
+#ifdef HAVE_LIBPNG
 void sdl_screenshot(int topwindow){
     gchar *c; 
     int i;
     SDL_Surface *surface;
 
     if (!sdl) return;
-
+    dbg("sdl_screenshot\n");
 
     if (topwindow){
         struct window *win;
@@ -1293,12 +1466,13 @@ void sdl_screenshot(int topwindow){
         struct stat st;
 
         if (ctest)
-            c=g_strdup_printf("%s/snap%d.png",ctest->directory,i);
+            c=g_strdup_printf("%s/snap%c%d.png",ctest->directory, aband->bandchar, i);
         else
-            c=g_strdup_printf("%s/tucnak/snap%d.png",getenv("HOME"),i);
+            c=g_strdup_printf("%s/tucnak/snap%d.png",getenv("HOME"), i);
         if (stat(c, &st)!=0) break;
         g_free(c);
     }
+    dbg("file=%s\n", c);
     
     if (do_png_save(surface, c)){
         log_addf(VTEXT(T_CANT_WRITE_S), c);
@@ -1308,5 +1482,46 @@ void sdl_screenshot(int topwindow){
 
     if (surface!=sdl->screen) SDL_FreeSurface(surface);
 }
+#endif
 
+extern char font_vga[4096];
+void sdl_font_save(){
+#ifdef HAVE_LIBPNG
+    SDL_Surface *surface;
+    int neww=13, newh=24, newwd=20, newhd=30;
+    int i, j, k, x, y, c, fx, fy;
+    char *fc;
+
+
+    surface=SDL_CreateRGBSurface(SDL_SWSURFACE, newwd*16, newhd*16, sdl->bpp,                               
+              sdl->screen->format->Rmask, sdl->screen->format->Gmask, sdl->screen->format->Bmask, 0);
+    SDL_FillRect(surface, NULL, sdl->gr[4]);
+
+    for (i=0; i<256; i++){
+        x = (i % 16) * newwd;
+        y = (i / 16) * newhd;
+        for (j=0; j<neww; j++){
+            fx = (j * 9)/neww;
+            if (fx==8) fx--;
+//            dbg("j=%d fx=%d\n", j, fx);
+            for (k=0; k<newh; k++){
+                c = sdl->yellow;
+                fy = (k * 16)/newh;
+
+	            fc=font_vga+ ((unsigned char)i)*16;
+                fc+=fy;
+                c = 0;
+                if (*fc & (1<<(7-fx))) c = sdl->gr[15];
+
+
+                fast_putpixel(surface, x+j, y+k, c);
+            }
+        }
+    }
+
+    do_png_save(surface, "font13x24.png");
+
+    SDL_FreeSurface(surface);
+#endif    
+}
 #endif
